@@ -21,7 +21,9 @@ use core::{
     sync::atomic::{AtomicPtr, AtomicUsize, Ordering},
 };
 
-use crate::{unreachable_unchecked, vtable::HasDropVt, Dyn, IStable, IntoDyn};
+use crate::{
+    unreachable_unchecked, vtable::HasDropVt, AnonymRef, AnonymRefMut, Dyn, IStable, IntoDyn,
+};
 
 use super::{
     vec::{ptr_add, ptr_diff, Vec, VecInner},
@@ -203,8 +205,9 @@ impl<T, Alloc: IAlloc> Arc<T, Alloc> {
     /// # Safety
     /// If used carelessly, this can cause mutable references and immutable references to the same value to appear,
     /// causing undefined behaviour.
+    #[rustversion::attr(since(1.86), const)]
     pub unsafe fn get_mut_unchecked(this: &mut Self) -> &mut T {
-        unsafe { this.ptr.as_mut() }
+        unsafe { this.ptr.ptr.as_mut() }
     }
 
     /// Returns the strong count.
@@ -249,6 +252,22 @@ impl<T, Alloc: IAlloc> Arc<T, Alloc> {
         unsafe { Self::get_mut_unchecked(self) }
     }
 
+    /// Returns a mutable reference to this `Arc`'s value, cloning that value into a new `Arc` if [`Self::get_mut`] would have failed.
+    pub fn make_mut_and_get_alloc(&mut self) -> (&mut T, &Alloc)
+    where
+        T: Clone,
+        Alloc: Clone,
+    {
+        if !Self::is_unique(self) {
+            *self = Self::new_in(
+                T::clone(self),
+                unsafe { self.ptr.prefix().alloc.assume_init_ref() }.clone(),
+            );
+        }
+        let (prefix, inner) = unsafe { self.ptr.split_mut() };
+        (inner, unsafe { prefix.alloc.assume_init_ref() })
+    }
+
     /// Whether or not `this` is the sole owner of its data, including weak owners.
     pub fn is_unique(this: &Self) -> bool {
         Self::strong_count(this) == 1 && Self::weak_count(this) == 1
@@ -420,9 +439,10 @@ impl<T, Alloc: IAlloc> ArcSlice<T, Alloc> {
         self.len() == 0
     }
     /// Returns a borrow to the slice.
+    #[rustversion::attr(since(1.86), const)]
     pub fn as_slice(&self) -> &[T] {
         let start = self.inner.start;
-        unsafe { core::slice::from_raw_parts(start.as_ptr(), self.len()) }
+        unsafe { core::slice::from_raw_parts(start.ptr.as_ptr(), self.len()) }
     }
     /// Returns a mutable borrow to the slice if no other references to it may exist.
     pub fn as_slice_mut(&mut self) -> Option<&mut [T]> {
@@ -432,9 +452,10 @@ impl<T, Alloc: IAlloc> ArcSlice<T, Alloc> {
     /// Returns a mutable borrow to the slice.
     /// # Safety
     /// This can easily create aliased mutable references, which would be undefined behaviour.
+    #[rustversion::attr(since(1.86), const)]
     pub unsafe fn as_slice_mut_unchecked(&mut self) -> &mut [T] {
         let start = self.inner.start;
-        unsafe { core::slice::from_raw_parts_mut(start.as_ptr(), self.len()) }
+        unsafe { core::slice::from_raw_parts_mut(start.ptr.as_ptr(), self.len()) }
     }
     /// Returns the strong count to the slice.
     pub fn strong_count(this: &Self) -> usize {
@@ -722,8 +743,11 @@ impl<T, Alloc: IAlloc> Drop for WeakSlice<T, Alloc> {
 pub use super::string::{ArcStr, WeakStr};
 
 impl<T, Alloc: IAlloc> crate::IPtr for Arc<T, Alloc> {
-    unsafe fn as_ref<U: Sized>(&self) -> &U {
-        self.ptr.cast().as_ref()
+    unsafe fn as_ref(&self) -> AnonymRef<'_> {
+        AnonymRef {
+            ptr: self.ptr.ptr.cast(),
+            _marker: PhantomData,
+        }
     }
 }
 impl<T, Alloc: IAlloc> crate::IPtrClone for Arc<T, Alloc> {
@@ -733,12 +757,18 @@ impl<T, Alloc: IAlloc> crate::IPtrClone for Arc<T, Alloc> {
 }
 
 impl<T, Alloc: IAlloc> crate::IPtrTryAsMut for Arc<T, Alloc> {
-    unsafe fn try_as_mut<U: Sized>(&mut self) -> Option<&mut U> {
-        Self::get_mut(self).map(|r| unsafe { core::mem::transmute::<&mut T, &mut U>(r) })
+    unsafe fn try_as_mut(&mut self) -> Option<AnonymRefMut<'_>> {
+        Self::is_unique(self).then(|| AnonymRefMut {
+            ptr: self.ptr.ptr.cast(),
+            _marker: PhantomData,
+        })
     }
 }
 impl<T, Alloc: IAlloc> crate::IPtrOwned for Arc<T, Alloc> {
-    fn drop(this: &mut core::mem::ManuallyDrop<Self>, drop: unsafe extern "C" fn(&mut ())) {
+    fn drop(
+        this: &mut core::mem::ManuallyDrop<Self>,
+        drop: unsafe extern "C" fn(AnonymRefMut<'_>),
+    ) {
         if unsafe { this.ptr.prefix() }
             .strong
             .fetch_sub(1, Ordering::Relaxed)
@@ -747,7 +777,10 @@ impl<T, Alloc: IAlloc> crate::IPtrOwned for Arc<T, Alloc> {
             return;
         }
         unsafe {
-            drop(this.ptr.cast().as_mut());
+            drop(AnonymRefMut {
+                ptr: this.ptr.ptr.cast(),
+                _marker: PhantomData,
+            });
             _ = Weak::<T, Alloc>::from_raw(this.ptr);
         }
     }
@@ -766,16 +799,18 @@ impl<T, Alloc: IAlloc> IntoDyn for Arc<T, Alloc> {
 }
 
 impl<T, Alloc: IAlloc> crate::IPtrOwned for Weak<T, Alloc> {
-    fn drop(this: &mut core::mem::ManuallyDrop<Self>, drop: unsafe extern "C" fn(&mut ())) {
+    fn drop(
+        this: &mut core::mem::ManuallyDrop<Self>,
+        _drop: unsafe extern "C" fn(AnonymRefMut<'_>),
+    ) {
         if unsafe { this.ptr.prefix() }
-            .strong
+            .weak
             .fetch_sub(1, Ordering::Relaxed)
             != 1
         {
             return;
         }
         unsafe {
-            drop(this.ptr.cast().as_mut());
             _ = Weak::<T, Alloc>::from_raw(this.ptr);
         }
     }
