@@ -81,10 +81,19 @@ where
 }
 use seal::Storage;
 mod seal {
+    use core::mem::MaybeUninit;
+
     use super::*;
     #[stabby::stabby]
     pub struct Storage<Size: Unsigned, Align: Alignment + Alignment> {
-        inner: <Align::Divide<Size> as IUnsignedBase>::Array<Align::AsUint>,
+        inner: MaybeUninit<<Align::Divide<Size> as IUnsignedBase>::Array<Align::AsUint>>,
+    }
+    impl<Size: Unsigned, Align: Alignment + Alignment> Storage<Size, Align> {
+        pub const fn zeroed() -> Self {
+            Self {
+                inner: MaybeUninit::zeroed(),
+            }
+        }
     }
 }
 
@@ -92,6 +101,7 @@ impl<Size: Unsigned, Align: Alignment + Alignment> Storage<Size, Align> {
     const fn as_ptr(&self) -> *const u8 {
         self as *const Self as *const _
     }
+    #[rustversion::attr(since(1.86), const)]
     fn as_mut_ptr(&mut self) -> *mut u8 {
         self as *mut Self as *mut _
     }
@@ -237,27 +247,23 @@ where
     /// Construct the `Ok` variant.
     #[allow(non_snake_case)]
     pub fn Ok(value: Ok) -> Self {
-        let mut storage = core::mem::MaybeUninit::zeroed();
+        let mut storage = Storage::zeroed();
+        let storage_ptr = &mut storage as *mut _;
         unsafe {
-            let storage_ptr = storage.as_mut_ptr();
             Self::ok_ptr_mut(storage_ptr).write(value);
             Self::det_ptr_mut(storage_ptr).write(Determinant::<Ok, Err>::ok(storage_ptr.cast()));
-            Self {
-                storage: storage.assume_init(),
-            }
+            Self { storage }
         }
     }
     /// Construct the `Err` variant.
     #[allow(non_snake_case)]
     pub fn Err(value: Err) -> Self {
-        let mut storage = core::mem::MaybeUninit::zeroed();
+        let mut storage = Storage::zeroed();
         unsafe {
-            let storage_ptr = storage.as_mut_ptr();
+            let storage_ptr = &mut storage as *mut _;
             Self::err_ptr_mut(storage_ptr).write(value);
             Self::det_ptr_mut(storage_ptr).write(Determinant::<Ok, Err>::err(storage_ptr.cast()));
-            Self {
-                storage: storage.assume_init(),
-            }
+            Self { storage }
         }
     }
     /// Converts to a standard [`Result`](core::result::Result) of immutable references to the variants.
@@ -465,9 +471,11 @@ where
     const unsafe fn err_unchecked(&self) -> &Err {
         &*Self::err_ptr(&self.storage)
     }
+    #[rustversion::attr(since(1.86), const)]
     unsafe fn ok_mut_unchecked(&mut self) -> OkGuard<'_, Ok, Err> {
         OkGuard { inner: self }
     }
+    #[rustversion::attr(since(1.86), const)]
     unsafe fn err_mut_unchecked(&mut self) -> ErrGuard<Ok, Err> {
         ErrGuard { inner: self }
     }
